# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: MIT-0
"""Get the OZI.build version currently in use."""
import os
import pathlib
import sys

if sys.version_info >= (3, 11):  # pragma: no cover
    import tomllib as toml
elif sys.version_info < (3, 11):  # pragma: no cover
    import tomli as toml

if __name__ == '__main__':
    with open(
        pathlib.Path(os.environ.get('MESON_SOURCE_ROOT', '.')) / 'pyproject.toml',
        'rb',
    ) as fh:
        pyproject = toml.load(fh)
    print(pyproject['build-system']['requires'][0].split('~=')[1])



# ozi/spec/python.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Python support specification metadata."""
from __future__ import annotations

import platform
from dataclasses import dataclass
from dataclasses import field
from datetime import date
from datetime import datetime
from datetime import timedelta
from datetime import timezone
from functools import cached_property
from typing import TYPE_CHECKING
from typing import Protocol
from typing import Sequence
from warnings import warn

from ozi_spec.base import Default
from ozi_spec.base import _FactoryDataclass

if TYPE_CHECKING:  # pragma: no cover
    from collections.abc import Mapping

pymajor, pyminor, pypatch = map(int, platform.python_version_tuple())
DATE_FORMAT = '%Y-%m-%d'
DEPRECATION_DELTA_WEEKS = 104


class _PythonSupport(_FactoryDataclass, Protocol):  # noqa: DC101
    deprecation_schedule: dict[int, str]
    major: str
    current_date: str

    @cached_property
    def _minor_versions(self: _PythonSupport) -> list[int]: ...  # noqa: DC103
    @property
    def bugfix_minor(self: _PythonSupport) -> int: ...  # noqa: DC102
    @property
    def bugfix(self: _PythonSupport) -> str: ...  # noqa: DC102
    @property
    def security1_minor(self: _PythonSupport) -> int: ...  # noqa: DC102
    @property
    def security1(self: _PythonSupport) -> str: ...  # noqa: DC102
    @property
    def security2_minor(self: _PythonSupport) -> int: ...  # noqa: DC102
    @property
    def security2(self: _PythonSupport) -> str: ...  # noqa: DC102
    @property
    def prerelease_minor(self: _PythonSupport) -> int | None: ...  # noqa: DC102
    @property
    def prerelease(self: _PythonSupport) -> str: ...  # noqa: DC102
    @property
    def classifiers(self: _PythonSupport) -> Sequence[tuple[str, str]]: ...  # noqa: DC102


@dataclass(frozen=True, slots=True, eq=True)
class PythonSupport(Default, _PythonSupport):
    """Python version support for the OZI toolchain."""

    deprecation_schedule: dict[int, str] = field(
        default_factory=lambda: {
            8: date(2024, 10, 1).strftime(DATE_FORMAT),
            9: date(2025, 10, 1).strftime(DATE_FORMAT),
            10: date(2026, 10, 1).strftime(DATE_FORMAT),
            11: date(2027, 10, 1).strftime(DATE_FORMAT),
            12: date(2028, 10, 1).strftime(DATE_FORMAT),
            13: date(2029, 10, 1).strftime(DATE_FORMAT),
            14: date(2030, 10, 1).strftime(DATE_FORMAT),
        },
    )
    major: str = field(init=False, default='3')
    current_date: str = field(
        init=False,
        compare=False,
        default_factory=lambda: datetime.now(tz=timezone.utc).date().strftime(DATE_FORMAT),
    )

    def __post_init__(self: _PythonSupport) -> None:
        """Warn the user if the python version is deprecated or pending deprecation.

        :raises: FutureWarning
        """
        python3_eol = (
            datetime.strptime(
                self.deprecation_schedule.get(
                    pyminor,
                    date(2008, 12, 3).strftime(DATE_FORMAT),
                ),
                DATE_FORMAT,
            )
            .replace(tzinfo=timezone.utc)
            .date()
        )
        ozi_support_eol = python3_eol - timedelta(weeks=DEPRECATION_DELTA_WEEKS)
        if datetime.now(tz=timezone.utc).date() > python3_eol:  # pragma: no cover
            warn(
                f'Python {pymajor}.{pyminor}.{pypatch} is not supported as of {python3_eol}.',
                FutureWarning,
            )
        elif datetime.now(tz=timezone.utc).date() > ozi_support_eol:  # pragma: no cover
            warn(
                f'Python {pymajor}.{pyminor}.{pypatch} support is pending deprecation '
                f'as of {ozi_support_eol}.',
                FutureWarning,
            )

    @cached_property
    def _minor_versions(self: _PythonSupport) -> list[int]:
        """List of currently supported Python minor version integers."""
        return sorted(
            [
                k
                for k, v in self.deprecation_schedule.items()
                if datetime.strptime(v, DATE_FORMAT).replace(tzinfo=timezone.utc)
                - timedelta(weeks=DEPRECATION_DELTA_WEEKS)
                > datetime.strptime(self.current_date, DATE_FORMAT).replace(
                    tzinfo=timezone.utc,
                )
            ],
        )[:4]

    @property
    def bugfix_minor(self: _PythonSupport) -> int:
        """Minor version integer for current bugfix Python release."""
        _, _, bugfix, *_ = self._minor_versions
        return bugfix

    @property
    def bugfix(self: _PythonSupport) -> str:
        """Version string for current bugfix Python release."""
        return '.'.join(map(str, (self.major, self.bugfix_minor)))

    @property
    def security1_minor(self: _PythonSupport) -> int:
        """Minor version integer for the most current security Python release."""
        _, security1, *_ = self._minor_versions
        return security1

    @property
    def security1(self: _PythonSupport) -> str:
        """Version string for the most current security Python release."""
        return '.'.join(map(str, (self.major, self.security1_minor)))

    @property
    def security2_minor(self: _PythonSupport) -> int:
        """Minor version integer for the second most current security Python release."""
        security2, *_ = self._minor_versions
        return security2

    @property
    def security2(self: _PythonSupport) -> str:
        """Version string for the second most current security Python release."""
        return '.'.join(map(str, (self.major, self.security2_minor)))

    @property
    def prerelease_minor(self: _PythonSupport) -> int | None:
        """Minor version integer for current prerelease Python release."""
        _, _, _, *prerelease = self._minor_versions
        return prerelease[0] if prerelease else None

    @property
    def prerelease(self: _PythonSupport) -> str:
        """Version string for current prerelease Python release."""
        if self.prerelease_minor:  # pragma: no cover
            return '.'.join(map(str, (self.major, self.prerelease_minor)))
        return ''  # pragma: defer to good-first-issue

    @property
    def classifiers(self: _PythonSupport) -> Sequence[tuple[str, str]]:  # pragma: no cover
        """Version classifiers for all currently supported Python releases."""
        classifiers = [
            ('Classifier', f'Programming Language :: Python :: {self.major} :: Only'),
            (
                'Classifier',
                f'Programming Language :: Python :: {self.security2}',
            ),
            (
                'Classifier',
                f'Programming Language :: Python :: {self.security1}',
            ),
            (
                'Classifier',
                f'Programming Language :: Python :: {self.bugfix}',
            ),
        ]
        if self.prerelease_minor:  # pragma: defer to spec
            classifiers += [
                ('Classifier', f'Programming Language :: Python :: {self.prerelease}'),
            ]
        return classifiers


_python_support = PythonSupport()


@dataclass(slots=True, frozen=True, eq=True)
class Support(Default):
    """Python implementation and version support info for OZI-packaged projects."""

    classifiers: Sequence[tuple[str, str]] = field(
        default_factory=lambda: _python_support.classifiers,
    )
    implementations: tuple[str, ...] = ('CPython',)
    metadata_version: str = '2.1'
    major: str = '3'
    prerelease: str = _python_support.prerelease
    bugfix: str = _python_support.bugfix
    security1: str = _python_support.security1
    security2: str = _python_support.security2
    deprecation_schedule: Mapping[int, str] = field(
        default_factory=lambda: _python_support.deprecation_schedule,
    )
    deprecation_delta_weeks: int = DEPRECATION_DELTA_WEEKS



# ozi_spec/get_github_publish_version.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Get the workflow version currently in use."""
import os
import pathlib

import yaml

if __name__ == '__main__':
    with open(
        pathlib.Path(os.environ.get('MESON_SOURCE_ROOT', '.')) / '.github/workflows/ozi.yml',
        'r',
    ) as fh:
        workflow = yaml.safe_load(fh)
    print(workflow['jobs']['publish']['steps'][1]['uses'].split('@')[1])



# ozi/spec/pkg.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Package specification metadata."""
from __future__ import annotations

from collections.abc import Sequence  # noqa: TCH003,TC003,RUF100
from dataclasses import dataclass
from dataclasses import field

from ozi_spec._license import SPDX_LICENSE_EXCEPTIONS
from ozi_spec._license import SPDX_LICENSE_MAP
from ozi_spec.base import Default


@dataclass(slots=True, frozen=True, eq=True, repr=False)
class PkgVersion(Default):
    """Versioning metadata.

    .. versionchanged:: OZI 1.5
       Default to `angular` semantic instead of `emoji`.

    .. versionchanged:: 0.23
       Default to `conventional` commit semantic, remove ``major_tags`` key.
    """

    semantic: str = 'conventional'
    allowed_tags: tuple[str, ...] = (
        'build',
        'chore',
        'ci',
        'docs',
        'feat',
        'fix',
        'perf',
        'style',
        'refactor',
        'test',
    )
    minor_tags: tuple[str, ...] = ('feat',)
    patch_tags: tuple[str, ...] = ('fix', 'perf', 'build')


@dataclass(slots=True, frozen=True, eq=True)
class PkgRequired(Default):
    """Required files for OZI project publishing."""

    root: tuple[str, ...] = (
        'README',
        'CHANGELOG.md',
        'pyproject.toml',
        'LICENSE.txt',
        '.gitignore',
    )

    source: tuple[str, ...] = ('__init__.py',)

    test: tuple[str, ...] = ()


@dataclass(slots=True, frozen=True, eq=True)
class PkgPattern(Default):
    """Regex patterns (or deferrals) for PKG-INFO headers."""

    name: str = r'^([A-Za-z]|[A-Za-z][A-Za-z0-9._-]*[A-Za-z0-9]){1,80}$'
    version: str = r'^(?P<prefix>v)?(?P<version>[^\\+]+)(?P<suffix>.*)?$'
    keywords: str = r'^(([a-z_]*[a-z0-9],)*{2,650})$'
    email: str = 'defer to RFC'
    license: str = 'defer to SPDX'
    license_id: str = 'defer to SPDX'
    license_exception_id = 'defer to SPDX'
    url: str = 'defer to IDNA'
    author: str = r'^((.+?)(?:,\s*|$)){1,128}$'
    summary: str = r'^(.*){1,255}$'
    copyright_head: str = r'^(.*){1,255}$'
    classifiers: str = r'^([\w\s]*\s\:\:\s)?'


@dataclass(slots=True, frozen=True, eq=True)
class PkgClassifiers(Default):
    """PKG-INFO default classifier metadata."""

    intended_audience: list[str] = field(default_factory=lambda: ['Other Audience'])
    typing: list[str] = field(default_factory=lambda: ['Typed'])
    environment: list[str] = field(default_factory=lambda: ['Other Environment'])
    language: list[str] = field(default_factory=lambda: ['English'])
    development_status: tuple[str] = ('1 - Planning',)


@dataclass(slots=True, frozen=True, eq=True)
class PkgInfo(Default):
    """PKG-INFO defaults metadata.

    .. versionchanged:: 0.23
       Remove deprecated required value ``Home-page``.

    """

    required: tuple[str, ...] = (
        'Author',
        'Author-email',
        'Description-Content-Type',
        'License',
        'Metadata-Version',
        'Name',
        'Summary',
        'Version',
    )
    classifiers: PkgClassifiers = PkgClassifiers()


@dataclass(slots=True, frozen=True, eq=True)
class License(Default):
    """Licensing specification metadata.

    .. versionchanged:: 0.10
       Add ``spdx_version`` key to track SPDX asset version.

    .. versionchanged:: 0.23
       Key ``exceptions`` is now a mapping from exception name to applicable licenses.
    """

    spdx_version: str = '3.26.0'
    ambiguous: dict[str, Sequence[str]] = field(
        default_factory=lambda: SPDX_LICENSE_MAP,
    )
    exceptions: dict[str, tuple[str, ...]] = field(
        default_factory=lambda: SPDX_LICENSE_EXCEPTIONS,
    )


@dataclass(slots=True, frozen=True, eq=True, repr=False)
class Pkg(Default):
    """Packaging specification metadata."""

    wheel: bool = True
    sdist: bool = True
    required: PkgRequired = PkgRequired()
    license: License = License()
    pattern: PkgPattern = PkgPattern()
    version: PkgVersion = PkgVersion()
    info: PkgInfo = PkgInfo()



# ozi/spec/ci.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Continuous integration specification."""
from __future__ import annotations

from collections.abc import Mapping  # noqa: TCH003,TC003,RUF100
from dataclasses import dataclass
from dataclasses import field

from ozi_spec.base import Default

@dataclass(slots=True, frozen=True, eq=True)
class Publish(Default):
    """Publishing patterns for packaged project."""

    include: tuple[str, ...] = ('*.tar.gz', '*.whl', 'sig/*')
    version: str = '@github_publish_version@'

@dataclass(slots=True, frozen=True, eq=True)
class Draft(Default):
    """Draft release patterns for packaged project."""

    version: str = '@github_draft_version@'

@dataclass(slots=True, frozen=True, eq=True)
class Release(Default):
    """Release patterns for packaged project."""

    version: str = '@github_release_version@'

@dataclass(slots=True, frozen=True, eq=True)
class GenerateProvenance(Default):
    """SLSA provenance generator metadata.

    .. versionadded:: 0.11.7
    """

    version: str = '@github_slsa_version@'

@dataclass(slots=True, frozen=True, eq=True)
class Checkpoint(Default):
    """Checkpoint suites to run."""

    suites: tuple[str, ...] = ('dist', 'lint', 'test')
    version: str = '@github_checkpoint_version@'

@dataclass(slots=True, frozen=True, eq=True)
class HardenRunnerEndpoints(Default):
    """Endpoints used in the GitHub CI workflow."""

    # fmt: off
    checkpoint: str = '@github_checkpoint_endpoints@'  # noqa: B950
    draft: str = '@github_draft_endpoints@'  # noqa: B950
    release: str = '@github_release_endpoints@'  # noqa: B950
    publish: str = '@github_publish_endpoints@'  # noqa: B950
    # fmt: on

@dataclass(slots=True, frozen=True, eq=True)
class HardenRunner(Default):
    """Github Step-Security harden runner."""

    version: str = '@github_harden_runner_version@'
    endpoints: HardenRunnerEndpoints = HardenRunnerEndpoints()

@dataclass(slots=True, frozen=True, eq=True)
class GithubActionPyPI(Default):
    """pypa/gh-action-pypi-publish"""

    version: str = '@github_action_pypi_publish_version@'

@dataclass(slots=True, frozen=True, eq=True)
class GithubMetadata(Default):
    """Github specific CI metadata"""

    harden_runner: HardenRunner = HardenRunner()
    gh_action_pypi_publish: GithubActionPyPI = GithubActionPyPI()
    provenance: GenerateProvenance = GenerateProvenance()



# ozi/spec/__init__.py
# Part of the OZI Project.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Unlicense
"""Specification API for OZI Metadata."""
from ozi_spec._spec import OZI
from ozi_spec._spec import Metadata
from ozi_spec._spec import Spec
from ozi_spec.base import Default
from ozi_spec.src import CommentPatterns

__all__ = (
    'CommentPatterns',
    'Default',
    'METADATA',
    'Metadata',
    'OZI',
    'Spec',
)

METADATA = Metadata()
